@@ -39,7 +39,7 @@
 #' \item{y: }{The y coordinate of the atom.}
 #' \item{z: }{The z coordinate of the atom.}
 #' \item{prediction_score: }{Contains the prediction score for each residue.}
-#' \item{auth_seq_id: }{Same as \code{label_seq_id}.}
+#' \item{auth_seq_id: }{Same as \code{label_seq_id}. But of type character.}
 #' \item{auth_comp_id: }{Same as \code{label_comp_id}.}
 #' \item{auth_asym_id: }{Same as \code{label_asym_id}.}
 #' \item{uniprot_id: }{The UniProt identifier of the predicted protein.}
@@ -50,11 +50,13 @@
 #' @import progress
 #' @import purrr
 #' @import tidyr
+#' @importFrom tibble tibble
 #' @importFrom utils download.file untar
 #' @importFrom readr read_tsv
 #' @importFrom stringr str_replace_all str_detect
 #' @importFrom curl has_internet
 #' @importFrom magrittr %>%
+#' @importFrom utils capture.output
 #' @export
 #'
 #' @examples
@@ -179,7 +181,7 @@ fetch_alphafold_prediction <- function(uniprot_ids = NULL,
           pb$tick()
         }
         readr::read_tsv(.x, col_names = FALSE, quote = "", show_col_types = FALSE, progress = FALSE) %>%
-          dplyr::filter(stringr::str_detect(X1, pattern = "^ATOM|^HETATM")) %>%
+          dplyr::filter(stringr::str_detect(X1, pattern = "^ATOM\\s+\\d|^HETATM\\s+\\d")) %>%
           dplyr::mutate(X2 = stringr::str_replace_all(X1, pattern = "\\s+", replacement = " ")) %>%
           tidyr::separate(X2,
             sep = " ",
@@ -232,7 +234,7 @@ fetch_alphafold_prediction <- function(uniprot_ids = NULL,
             y = as.numeric(.data$y),
             z = as.numeric(.data$z),
             prediction_score = as.numeric(.data$prediction_score),
-            auth_seq_id = as.numeric(.data$auth_seq_id)
+            auth_seq_id = .data$auth_seq_id
           ) %>%
           dplyr::mutate(score_quality = dplyr::case_when(
             .data$prediction_score > 90 ~ "very_good",
@@ -270,7 +272,6 @@ fetch_alphafold_prediction <- function(uniprot_ids = NULL,
       .x = batches,
       .f = ~ {
         # query information from database
-        if (!is.null(batches)) {
           query <- try_query(.x,
             type = "text/tab-separated-values",
             col_names = FALSE,
@@ -278,18 +279,15 @@ fetch_alphafold_prediction <- function(uniprot_ids = NULL,
             show_col_types = FALSE,
             progress = FALSE
           )
-        }
-        if (show_progress == TRUE & "tbl" %in% class(query)) {
+          
+          if (show_progress == TRUE) {
           pb$tick()
-        }
-        # if previous batch had a connection problem change batches to NULL, which breaks the mapping.
-        if (!"tbl" %in% class(query)) {
-          batches <<- NULL
-        }
+          }
+          
         # only proceed with data if it was correctly retrieved
         if ("tbl" %in% class(query)) {
           query %>%
-            dplyr::filter(stringr::str_detect(X1, pattern = "^ATOM|^HETATM")) %>%
+            dplyr::filter(stringr::str_detect(X1, pattern = "^ATOM\\s+\\d|^HETATM\\s+\\d")) %>%
             dplyr::mutate(X2 = stringr::str_replace_all(X1, pattern = "\\s+", replacement = " ")) %>%
             tidyr::separate(X2,
               sep = " ",
@@ -342,7 +340,7 @@ fetch_alphafold_prediction <- function(uniprot_ids = NULL,
               y = as.numeric(.data$y),
               z = as.numeric(.data$z),
               prediction_score = as.numeric(.data$prediction_score),
-              auth_seq_id = as.numeric(.data$auth_seq_id)
+              auth_seq_id = .data$auth_seq_id
             ) %>%
             dplyr::mutate(score_quality = dplyr::case_when(
               .data$prediction_score > 90 ~ "very_good",
@@ -350,11 +348,31 @@ fetch_alphafold_prediction <- function(uniprot_ids = NULL,
               .data$prediction_score > 50 ~ "low",
               .data$prediction_score <= 50 ~ "very_low"
             ))
+        } else {
+          query
         }
       }
     )
   }
+  
+  # catch any IDs that have not been fetched correctly
+  error_list <- query_result %>% 
+    purrr::keep(.p = ~ is.character(.x))
+  
+  error_table <- tibble::tibble(id = names(error_list),
+                                error = unlist(error_list)) %>% 
+    dplyr::distinct()
+  
+  if(nrow(error_table) != 0){
+    message("The following IDs have not be retrieved correctly.")
+    message(paste0(utils::capture.output(error_table), collapse = "\n"))
+  }
 
+  # only keep data in output
+  
+  query_result <- query_result %>% 
+    purrr::keep(.p = ~ !is.character(.x))
+  
   if (return_data_frame == FALSE) {
     return(query_result)
   } else {
