@@ -5,8 +5,9 @@
 #'
 #' @param data a data frame that contains at least sample names, grouping identifiers (precursor,
 #' peptide or protein) and log2 transformed intensities for each grouping identifier.
-#' @param sample a character column in the \code{data} data frame that contains the sample name.
-#' NOTE: If the overall distribution should be returned please do not provide the name of the
+#' @param sample an optional character or factor column in the \code{data} data frame that contains the 
+#' sample name. If the sample column is of type factor, the ordering is based on the factor
+#' levels. NOTE: If the overall distribution should be returned please do not provide the name of the
 #' sample column.
 #' @param grouping a character column in the \code{data} data frame that contains the grouping
 #' variables (e.g. peptides, precursors or proteins).
@@ -57,7 +58,7 @@ qc_intensity_distribution <- function(data,
     dplyr::distinct({{ sample }}, {{ grouping }}, {{ intensity_log2 }}) %>%
     tidyr::drop_na({{ intensity_log2 }})
 
-  if (!missing(sample)) {
+  if (!missing(sample) && class(dplyr::pull(input, {{ sample }})) != "factor") {
     input <- input %>%
       dplyr::mutate({{ sample }} := factor({{ sample }},
         levels = unique(stringr::str_sort({{ sample }}, numeric = TRUE))
